﻿-- Author      : Sidrus
-- Create Date : 7/4/2009 11:12:03 PM
do
	local LO_transactions;
	local LO_dateSort = LoanOfficerConsts.SORT_DESC;
	local LO_goldSort = LoanOfficerConsts.SORT_DESC;

	-- HELPERS --
	local function TransactionLogFrame_ResetForm()
		for line=1,LoanOfficerConsts.TRANS_LOG_LINES do
			lineObj = getglobal("TransLogEntry"..line);
			lineObj:Hide();
		end
		txtTransLogAccount:SetText("");
		LO_transactions = nil;
		LO_HideFrame("TransactionLogFrame");
		LO_ShowFrame("MainFrame");
		LO_dateSort = LoanOfficerConsts.SORT_DESC;
	end
	local function GetLineObjects(line)
		local lineObj      = getglobal("TransLogEntry"..line);
		local lineObjDate  = getglobal("TransLogEntry"..line.."Date");
		local lineObjAmt   = getglobal("TransLogEntry"..line.."Amount");
		local lineObjNotes = getglobal("TransLogEntry"..line.."Notes");
		return lineObj, lineObjDate, lineObjAmt, lineObjNotes;
	end
	local function GetTransData(account)	
		-- fill the empty table with our transaction log
		LO_transactions = App:GetTransactions(account)
		
		-- ensure we got a transaction list before proceeding
		if (LO_transactions == nil) then
			App:Print(string.format("Failed to get the transaction list for: %s", account));
			return {};
		end
	end
	
	-- SORTING --
	local function AscendingDateSorter(a, b)
		return (a.date < b.date);
	end
	local function DescendingDateSorter(a, b)
		return (a.date > b.date);
	end
	local function SortDate()
		if (LO_transactions == nil or type(LO_transactions) ~= "table") then
			-- invalid state
			App:PrintDebug("Date sorter received invalid operand.");
		else
			-- sort the table
			if (LO_dateSort == LoanOfficerConsts.SORT_ASC) then
				table.sort(LO_transactions, AscendingDateSorter);
			else
				table.sort(LO_transactions, DescendingDateSorter);
			end
		end
	end
	function LO_TransLogToggleDateSort()
		if (LO_dateSort == LoanOfficerConsts.SORT_ASC) then
			LO_dateSort = LoanOfficerConsts.SORT_DESC;
		else
			LO_dateSort = LoanOfficerConsts.SORT_ASC;
		end
		
		-- refresh the view
		SortDate();
		transLogScroll_Update();
	end
	
	local function AscendingGoldSorter(a, b)
		return (abs(a.amount) < abs(b.amount));
	end
	local function DescendingGoldSorter(a, b)
		return (abs(a.amount) > abs(b.amount));
	end
	local function SortGold()
		if (LO_transactions == nil or type(LO_transactions) ~= "table") then
			-- invalid state
			App:PrintDebug("Gold sorter received invalid operand.");
		else
			-- sort the table
			if (LO_goldSort == LoanOfficerConsts.SORT_ASC) then
				table.sort(LO_transactions, AscendingGoldSorter);
			else
				table.sort(LO_transactions, DescendingGoldSorter);
			end
		end
	end
	function LO_TransLogToggleGoldSort()
		if (LO_goldSort == LoanOfficerConsts.SORT_ASC) then
			LO_goldSort = LoanOfficerConsts.SORT_DESC;
		else
			LO_goldSort = LoanOfficerConsts.SORT_ASC;
		end
		
		-- refresh the view
		SortGold();
		transLogScroll_Update();
	end
	
	-- SCROLL EVENTS --
	function transLogScroll_OnVerticalScroll()
		transLogScroll_Update();
	end
	function transLogScroll_Update()
		-- can't work on an empty table
		if (LO_transactions == nil) then
			return;
		end
	
	    -- these are declared out here to prevent re-declaring them
	    -- over and over in the loop.
		local maxItems = #LO_transactions;
		local line;
		local lineplusoffset;
		local lineObj, lineObjDate, lineObjAmt, lineObjNotes;
		local trans;

		-- call the base class function	   
		FauxScrollFrame_Update(transLogScroll, maxItems, LoanOfficerConsts.TRANS_LOG_LINES, LoanOfficerConsts.TRANS_LOG_LINE_HEIGHT);
		
		-- loop through our table and print the values
		for line=1,LoanOfficerConsts.TRANS_LOG_LINES do
			lineObj, lineObjDate, lineObjAmt, lineObjNotes = GetLineObjects(line);
			lineplusoffset = line + (FauxScrollFrame_GetOffset(transLogScroll) or 0);
	       
			if lineplusoffset <= maxItems then
				-- Get a copy of the transaction data
				trans = LO_transactions[lineplusoffset];
	                        
				-- Configure the font color
				if (trans.amount < 0) then  
					lineObjDate:SetFontObject("LO_GameFontGreenLeft");
					lineObjAmt:SetFontObject("LO_GameFontGreenLeft");
					lineObjNotes:SetFontObject("LO_GameFontGreenLeft");
				else
					lineObjDate:SetFontObject("LO_GameFontRedLeft");
					lineObjAmt:SetFontObject("LO_GameFontRedLeft");
					lineObjNotes:SetFontObject("LO_GameFontRedLeft");
				end
	            
				-- Print the button text and show
				lineObjDate:SetText(trans.date);
				lineObjAmt:SetText(LO_FormatGold(trans.amount));
				if (trans.notes ~= "") then
					lineObjNotes:SetText(trans.notes);
				end
	            
	            -- make sure the line item is visible
				lineObj:Show();
			else
				-- this line item isn't used for this update
				lineObj:Hide();
			end
		end
	end

	-- BUTTON HANDLERS --
	function cmdTransLogClose_OnClick()
		TransactionLogFrame_ResetForm();
	end
	function cmdLoadAcctTransLog_OnClick()
		local LO_transAccount = (txtTransLogAccount:GetText() or "");    
		if (LO_transAccount~="") then
			local g,s,c = App:CalculateBalance(LO_transAccount);
			lblTransLogBalance:SetFormattedText("Current Balance: %s", LO_FormatParsedGold(g,s,c));			
			GetTransData(LO_transAccount);	        
			transLogScroll_Update();
		end
	end

	-- INITIALIZATION --
	function TransactionLogFrame_OnLoad()
		TransactionLogFrame_ResetForm();
	end
end